#!/usr/local/bin/perl

package List;
use Moose;
use Moose::Util::TypeConstraints;

has 'value' => (is => 'rw', isa => 'Int');

coerce 'List',
    => from 'Int',
    => via { List->new(value => $_[0]) };

has 'next' => (
    is        => 'rw',
    isa       => 'List',
    coerce    => 1,
    predicate => 'has_next',
);

sub iterator {
    List::Iterator->new( list => shift );
}

package Iterator;
use Moose::Role;

requires qw/has_next next/;

package List::Iterator;
use Moose;

with 'Iterator';

has 'list' => (
    is        => 'rw',
    isa       => 'Undef|List',
);

sub next {
    my $self = shift;
    my $value = $self->list->value;

    $self->list->has_next
        ? $self->list( $self->list->next )
        : $self->list( undef );

    return $value;
}

sub has_next {
    defined shift->list;
}

package main;

my $list = List->new(value => 1);
$list->next(2);
$list->next->next(3);
$list->next->next->next(4);
$list->next->next->next->next(5);

my $iter = $list->iterator;
while ($iter->has_next) {
    printf "%d\n",$iter->next;
}
